%
%  Subject : Approximation of functions using narrow-base trial functions 
%
%  Example Ej_3_1_4
%
%         Wl = Galerkin
%         Nm = piecewise linear
%         see Zienkiewicz & Morgan page 99
% 

% Number of elements to be used
Nx = 20;

% grid points
xnod  = (0:Nx)'/Nx;
icone = [(1:Nx)',(2:Nx+1)'];

% function to be approximated
phi = ffun_Ej_3_1_4(xnod(:,1));

% function used to match end values
%psi = 0*(phi(1)+(phi(end)-phi(1))*xe);

[numel,nen]=size(icone);
Ke = zeros(numel,2,2);
fe = zeros(numel,2);

for k=1:numel
    psi = (0:10)'/10; 
    node1 = icone(k,1);
    node2 = icone(k,2);
    xx  = psi*(xnod(node2,1)-xnod(node1,1))+xnod(node1,1);        
    Na = shape_Ej_3_1_4(xx,xnod(node2,1),xnod(node1,1));    
    Nb = shape_Ej_3_1_4(xx,xnod(node1,1),xnod(node2,1));
    Ke(k,1,1) = Ke(k,1,1) + trapz(xx,Na.*Na);    
    Ke(k,1,2) = Ke(k,1,2) + trapz(xx,Na.*Nb);        
    Ke(k,2,1) = Ke(k,2,1) + trapz(xx,Nb.*Na);    
    Ke(k,2,2) = Ke(k,2,2) + trapz(xx,Nb.*Nb);            
    fe(k,1) = fe(k,1) + trapz(xx,Na.*ffun_Ej_3_1_4(xx)); 
    fe(k,2) = fe(k,2) + trapz(xx,Nb.*ffun_Ej_3_1_4(xx));     
end

% gather Ke and fe in Kg and fg
Kg = zeros(Nx+1,Nx+1);
fg = zeros(Nx+1,1);
for k=1:numel
    node1 = icone(k,1);
    node2 = icone(k,2);
    Kg(node1,node1)=Kg(node1,node1)+Ke(k,1,1);
    Kg(node1,node2)=Kg(node1,node2)+Ke(k,1,2);
    Kg(node2,node1)=Kg(node2,node1)+Ke(k,2,1);
    Kg(node2,node2)=Kg(node2,node2)+Ke(k,2,2);    
    fg(node1,1) = fg(node1,1) + fe(k,1);
    fg(node2,1) = fg(node2,1) + fe(k,2);
end

% solver
a = Kg\fg;

% Visualization
figure(1);clf;
x = xnod(:,1);
xe = 0.5*(x(1:end-1)+x(2:end));
plot(x,ffun_Ej_3_1_4(x),'m',xe,zeros(size(xe)),'r+',x,zeros(size(x)),'bo',x,a,'k*')
Xh=[]; Yh=[];
xlabel('X');ylabel('phi');title('1D piecewise linear - Galerkin aproximation ')
for k=1:Nx
    Xh = [Xh,[x(k),x(k+1)]];
    Yh = [Yh,[a(k),a(k+1)]];
end
figure(1);hold on;line(Xh,Yh);

