%
%  Tema II : TECNICAS DE DISCRETIZACION
%
%  Solucion de un ODE de segundo orden mediante Residuos ponderados
%  usando funciones de prueba globales y pesos del tipo:
%  [a] Colocacion puntual
%  [b] Galerkin
%

% Dominio a resolver
x     = (0:100)'/100;
nn    = length(x);

% Numero de terminos a utilizar
M = 2;

% Calculo de los coeficientes de la matriz y del vector residuo
% Galerkin
K_gal = zeros(M,M); f_gal=zeros(M,1);
for m=1:M,
  K_gal(m,m) = 1/2*(1+m^2*pi^2);
  f_gal(m)   = (-1)^m/(m*pi);
end
% calculo de los coeficientes de la expansion 
a_gal = K_gal\f_gal;

% Colocacion
K_col = zeros(M,M); f_col=zeros(M,1);
% puntos de colocacion
dx = (max(x)-min(x))/(M+1); xc=zeros(M,1);
for k=1:M,
  xc(k) = min(x)+k*dx;
end

for l=1:M,
 for m=1:M,
  K_col(l,m) = (1+m^2*pi^2)*sin(m*pi*xc(l));
  f_col(l)   = -xc(l);
 end
end
% calculo de los coeficientes de la expansion 
a_col = K_col\f_col;

psi = x;
phi_gal = psi;
phi_col = psi;
for m=1:M,
  phi_gal = phi_gal + a_gal(m)*sin(m*pi*x);
  phi_col = phi_col + a_col(m)*sin(m*pi*x);
end  

% Solucion exacta

phi_ex = (1/(exp(1) - exp(-1)))*(exp(x)-exp(-x));

% Graficacion de la aproximacion
figure(1);clg;plot(x,x-phi_ex,x,x-phi_gal,x,x-phi_col)
figure(2);clg;plot(x,abs(phi_ex-phi_col),x,abs(phi_ex-phi_gal))
e_gal = trapz(x,abs(phi_ex-phi_gal));
e_col = trapz(x,abs(phi_ex-phi_col));
gtext([' |e|_Gal = ' num2str(e_gal)])
gtext([' |e|_Col = ' num2str(e_col)])

return
